!***********************************************************************
!*****************************  KEK, High Energy Accelerator Research  *
!*****************************  Organization                           *
!* u c x y z  _ p h a n t o m*                                         *
!*****************************     EGS5.0 USER CODE - 19 Nov 2012/1300 *
!***********************************************************************
!* This is a general User Code based on the cg geometry scheme.        *
!***********************************************************************
!                                                                      *
!  PROGRAMMERS:  H. Hirayama                                           *
!                Radiation Science Center                              *
!                Applied Science Laboratory                            *
!                KEK, High Energy Accelerator Research Organization    *
!                1-1, Oho, Tsukuba, Ibaraki, 305-0801                  *
!                Japan                                                 *
!                                                                      *
!                E-mail:      hideo.hirayama@kek.jp                    *
!                Telephone:   +81-29-864-5489                          *
!                Fax:         +81-29-864-1993                          *
!                Based on xyzdos.mor.                                  *
!                                                                      *
! 09AUG2004: CDF creation parts are corrected.    H. Hirayama          *
!***********************************************************************
!***********************************************************************
! The ucxyz_phantom.f User Code requires a data-input file             *
! (e.g., ucxyz_phantom.data) that is read by subroutine getvoxel (with *
! instructions in its header).  The following shows the geometry for   *
! ucxyz_phantom.data.                                                  *
! Trajectory data can be defined for PICT32 or CGview by setting       *
! npreci 1, 2 or 3, respectively.                                      *
! This user code corresponds to ucphantom_rec.mor for egs4.            *
! Use Ranlux random number generator.                                  *
!***********************************************************************
!                                                                      *
!             ---------------------------------------------------      *
!             3-Dimensional X-Y-Z Geometry (ucxyz_pahtom example)      *
!             ---------------------------------------------------      *
!                                                                      *
!                     X (Y into page)                                  *
!                     ^                                                *
!                     |                                                *
!                     |                                                *
!                     | Outer vacuum region                            *
!                +----+---+---+-   -+---+----+    15.0cm               *
!        Vacuum  |    |   |   |     |   |    |                         *
!        region  +Air +H2O|Water (H2O)  |Air |                         *
!                |    |   |   |     |H2O|    |                         *
!                +----+---+---+-   -+---+----+--   0.5                 *
!                |    |   |   |     |   |    |                         *
!                |Air |H2O|H2O|     |H2O|Air |                         *
! 1.253MeV photon|    |   |   |     |   |    |                         *
!      =========>+----+---+---+-   -+---+----+-----> Z                 *
! photons      -5.0  0.0 1.0       19.0 20.0                           *
!                                          25.0                        *
!                                                                      *
!                                                                      *
!***********************************************************************
!23456789|123456789|123456789|123456789|123456789|123456789|123456789|12
!
!-----------------------------------------------------------------------
!------------------------------- main code -----------------------------
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
! Step 1: Initialization
!-----------------------------------------------------------------------

      implicit none

!     ------------
!     EGS5 COMMONs
!     ------------
      include 'include/egs5_h.f'                ! Main EGS "header" file

      include 'include/egs5_bounds.f'
      include 'include/egs5_edge.f'
      include 'include/egs5_elecin.f'
      include 'include/egs5_media.f'
      include 'include/egs5_misc.f'
      include 'include/egs5_stack.f'
      include 'include/egs5_thresh.f'
      include 'include/egs5_uphiot.f'
      include 'include/egs5_useful.f'
      include 'include/egs5_usersc.f'
      include 'include/randomm.f'

!     ----------------------
!     Auxiliary-code COMMONs
!     ----------------------
      include 'auxcommons/aux_h.f'   ! Auxiliary-code "header" file

      include 'auxcommons/edata.f'
      include 'auxcommons/etaly1.f'
      include 'auxcommons/geoxyz.f'
      include 'auxcommons/instuf.f'
      include 'auxcommons/lines.f'
      include 'auxcommons/nfac.f'
      include 'auxcommons/voxel.f'
      include 'auxcommons/watch.f'

      common/score/                             ! Variables to score
     *             depe(LIMAX,LJMAX,LKMAX),faexp,fexps,maxpict
      real*8 depe,faexp,fexps
      integer maxpict

!**** real*8                                                 ! Arguments
      real*8 etot,totke
!**** integer
      integer ifto

!**** real*8                                           ! Local variables
      real*8
     * amass,areac,availke,depthl,depths,dis,disair,ei0,ekin,elow,eup,
     * phai0,phai,radma2,rnnow,sinth,sposi,tnum,w0,wimin,wtin,wtsum,
     * xhbeam,xpf,yhbeam,ypf

      real*8 bsfa,bsferr,faexps,faexp2s,faexrr,fexpss,fexps2s,fexerr,
     *       faexpa,fexpsa

      real*8
     * depeh(LIMAX,LJMAX,LKMAX),depeh2(LIMAX,LJMAX,LKMAX),
     * dose(LIMAX,LJMAX,LKMAX),doseun(LIMAX,LJMAX,LKMAX)

      real
     * tarray(2),tt,tt0,tt1,cputime,etime

      integer
     * i,ii,iii,icases,idin,idose,ie,ipage,irl,j,jhist,jj,jl,ju,k,
     * kkk,nlist,nperpg

!     ----------
!     Open files
!     ----------
!-----------------------------------------------------------------
!     Units 7-26 are used in pegs and closed.  It is better not
!     to use as output file. If they are used must be re-open after
!     getxyz etc. Unit for pict must be 39.
!-----------------------------------------------------------------

      open(6,file='egs5job.out',status='unknown')
      open(4,FILE='egs5job.inp',STATUS='old')
      open(39,FILE='egs5job.pic',STATUS='unknown')

!     ====================
      call counters_out(0)
!     ====================

!-----------------------------------------------------------------------
! Step 2: pegs5-call
!-----------------------------------------------------------------------
      ifto = 6      ! Output unit in getvoxel
!     ===================
      call getvoxel(ifto)
!     ===================

!     ------------------------------
!     Run PEGS5 before calling HATCH
!     ------------------------------
      write(6,*) 'PEGS5-call comes next'

!     ==========
      call pegs5
!     ==========

!-----------------------------------------------------------------------
! Step 3: Pre-hatch-call-initialization
!-----------------------------------------------------------------------
!-----------------------------------------------
!     Define pict data mode.
!-----------------------------------------------
!     npreci 1: for PICT32
!            2: for CGview
!            3: for CGview in free format
      npreci=3

      if(npreci.eq.3) write(39,fmt="('GSTA-FREE-TIME')")
      if(npreci.eq.2) write(39,fmt="('GSTA-TIME')")
      write(39,fmt="('SLAB')")
      write(39,fmt="(I6)") imax+1
      write(39,fmt="(I6)") jmax+1
      write(39,fmt="(I6)") kmax+1
      write(39,fmt="(4F15.4)") (xbound(j),j=1,imax+1)
      write(39,fmt="(4F15.4)") (ybound(j),j=1,jmax+1)
      write(39,fmt="(4F15.4)") (zbound(j),j=1,kmax+1)
      write(39,fmt="('GEND')")
        
      write(39,fmt="('MSTA')")
      write(39,fmt="(I4)") nreg
      write(39,fmt="(15I4)") (med(i),i=1,nreg)
      write(39,fmt="('MEND')")

!     --------------------------------------------------------
!     Random number seeds.  Must be defined before call hatch
!     or defaults will be used.  inseed (1- 2^31)
!     --------------------------------------------------------
      luxlev = 1
      inseed=1
      write(6,100) inseed
100   FORMAT(/,' inseed=',I12,5X,
     *         ' (seed for generating unique sequences of Ranlux)')

!     =============
      call rluxinit  ! Initialize the Ranlux random-number generator
!     =============

!-----------------------------------------------------------------------
! Step 4:  Determination-of-incident-particle-parameters
!-----------------------------------------------------------------------

!-------------------------------------------------
!     Define source position from phantom surface.
!-------------------------------------------------
!     Source position from phantom surface in cm.
      sposi=10.0

      iqin=0             ! Incident charge - photons
      ekein=1.253        ! Kinetic energy of source photon
      etot=ekein + abs(iqin)*RM
      xin=0.D0    
      yin=0.D0    
      zin=-sposi 
      uin=0.D0    
      vin=0.D0    
      win=1.D0    
      wtin=1.D0

!---------------------------------------------
!     Half width and height at phantom surface
!---------------------------------------------
!     X-direction half width of beam at phantom surface in cm.
      xhbeam=1.0
!     Y-direction half height of beam at phantom surface in cm.
      yhbeam=1.0
      radma2=xhbeam*xhbeam+yhbeam*yhbeam
      wimin=sposi/dsqrt(sposi*sposi+radma2)

!-----------------------------------------------------------------------
! Step 5:   hatch-call
!-----------------------------------------------------------------------
      emaxe = 0.D0 ! dummy value to extract min(UE,UP+RM).

      write(6,110)
110   format(/' Call hatch to get cross-section data')
!     ------------------------------
!     Open files (before HATCH call)
!     ------------------------------
      open(UNIT=KMPI,FILE='pgs5job.pegs5dat',STATUS='old')
      open(UNIT=KMPO,FILE='egs5job.dummy',STATUS='unknown')

      write(6,120)
120   FORMAT(/,' HATCH-call comes next',/)

!     ==========
      call hatch
!     ==========

!     ------------------------------
!     Close files (after HATCH call)
!     ------------------------------
      close(UNIT=KMPI)
      close(UNIT=KMPO)

!-----------------------------------------------------------------------
!     Output medium and region information to file for calculation mode.
!-----------------------------------------------------------------------
      write(6,*) ' Quantities associated with each media:' 
      do j=1,nmed 
        write(6,130) (media(i,j),i=1,24)
130     FORMAT(/,1X,24A1)
        write(6,140) rhom(j),rlcm(j)
140     FORMAT(5X,' Rho=',G15.7,' g/cm**3     RLC=',G15.7,' cm')
        write(6,150) ae(j),ue(j),ap(j),up(j)
150     FORMAT(5X,' AE=',G15.7,' MeV    UE=',G15.7,' MeV'/ 5X,' AP=',G
     *  15.7,' MeV    UP=',G15.7,' MeV')
      end do

      write(6,160)
160   FORMAT(/' Information of medium and cut-off for central region')
      i=imax/2+1
      j=jmax/2+1
      do k=1,kmax
        irl=1+i+(j-1)*imax+(k-1)*ijmax
        if (med(irl).eq.0) then
          write(6,170) k,irl
170       FORMAT(' Medium(',I3,'-th z bin, region:',I5,')= Vacuum')
        else
          write(6,180) k,irl,(media(ii,med(irl)),ii=1,24),
     *                 ecut(irl),pcut(irl),rhor(irl)
180       FORMAT(' Medium(',I3,'-th z bin, region:',I5,
     *           ')=',24A1,/5X,'ECUT=',G10.5,' MeV, PCUT=',
     *           G10.5, ' MeV, density=',F10.3)
        end if
      end do

!-----------------------------------------------------------------------
! Step 6:  Initialization-for-howfar
!-----------------------------------------------------------------------
!       Initialization is done inside getvoxel

!-----------------------------------------------------------------------
! Step 7:  Initialization-for-ausgab
!-----------------------------------------------------------------------
      ncount = 0
      ilines = 0
      nwrite = 10
      nlines = 25
      idin = -1
      totke = 0.
      wtsum = 0.

!---------------------
!     Clear variables
!---------------------
!     Zero the dose 
      do k=1,kmax
        do j=1,jmax
          do i=1,imax
            depe(i,j,k)=0.D0
            depeh(i,j,k)=0.D0
            depeh2(i,j,k)=0.D0
          end do
        end do
      end do

      faexp=0.D0
      faexps=0.D0
      faexp2s=0.D0
      fexps=0.D0
      fexpss=0.D0
      fexps2s=0.D0
      
!     =========================
      call ecnsv1(0,nreg,totke)
      call ntally(0,nreg)
!     =========================

!--------------------------
!     History number 
!--------------------------
!     History number
      ncases=100000
!     Maximum history number to write trajectory data
      maxpict=50

      write(39,fmt="('0    1')")

      write(6,190)
190   FORMAT(//,' ENERGY/COORDINATES/DIRECTION COSINES/ETC.',/,
     *        6X,'E',16X,'X',14X,'Y',14X,'Z'/
     *        1X,'U',14X,'V',14X,'W',9X,'IQ',4X,'IR',3X,'IARG',/)

!                        =======================
      if (iwatch .gt. 0) call swatch(-99,iwatch)
!                        =======================
 
      tt=etime(tarray)
      tt0=tarray(1)

!-----------------------------------------------------------------------
! Step 8:  Shower-call
!-----------------------------------------------------------------------

                                             ! -------------------------
      do jhist=1,ncases                      ! Start of CALL SHOWER loop
                                             ! -------------------------
        icases=jhist
!--------------------------------------
!       Determine direction (isotropic)
!--------------------------------------
200     call randomset(w0)
        win=w0*(1.0-wimin)+wimin
        call randomset(phai0)
        phai=pi*(2.0*phai0-1.0)
        sinth=dsqrt(1.D0-win*win)
        uin=dcos(phai)*sinth
        vin=dsin(phai)*sinth
        dis=sposi/win
        xpf=dis*uin
        ypf=dis*vin
        if (dabs(xpf).gt.xhbeam.or.dabs(ypf).gt.yhbeam) go to 200
        if (sposi.gt.zbound(2)-zbound(1)) then
          disair=(sposi-(zbound(2)-zbound(1)))/win
          xin=disair*uin
          yin=disair*vin
          zin=zbound(1)
        else
          xin=0.D0
          yin=0.D0
          zin=-sposi
        end if

        do i=1,imax
            if (xbound(i+1).gt.xin) go to 210
        end do

210     do j=1,jmax
          if (ybound(j+1).gt.yin) go to 220
        end do

!       -------------
!       Input region
!       -------------
220     k=1
        irin=1+i+(j-1)*imax

!       ----------------------
!       Select incident energy
!       ----------------------
        ekin = ekein
        wtsum = wtsum + wtin               ! Keep running sum of weights
        etot = ekin + iabs(iqin)*RM        ! Incident total energy (MeV)
        availke = etot + iqin*RM        ! Available K.E. (MeV) in system
        totke = totke + availke                 ! Keep running sum of KE

        latchi=0

!       ---------------------------------------------------
!       Print first NWRITE or NLINES, whichever comes first
!       ---------------------------------------------------
        if (ncount .le. nwrite .and. ilines .le. nlines) then
          ilines = ilines + 1
          write(6,230) etot,xin,yin,zin,uin,vin,win,iqin,irin,idin
230       FORMAT(7G15.7,3I5)
        end if

!       -----------------------------------------------------------
!       Compare maximum energy of material data and incident energy
!       -----------------------------------------------------------
        if(etot+(1-iabs(iqin))*RM.gt.emaxe) then 
          write(6,fmt="(' Stopped in MAIN.',
     1    ' (Incident kinetic energy + RM) > min(UE,UP+RM).')")
          stop
        end if

!       ----------------------------------------------------
!       Verify the normarization of source direction cosines
!       ----------------------------------------------------
        if(abs(uin*uin+vin*vin+win*win-1.0).gt.1.e-6) then
          write(6,fmt="(' Following source direction cosines are not', 
     1    ' normarized.',3e12.5)")uin,vin,win
          stop
        end if

!       =========================================================
        call shower (iqin,etot,xin,yin,zin,uin,vin,win,irin,wtin)
!       =========================================================

!----------------------------------
!      Sum variable and its square.
!----------------------------------

        do k=1,kmax
          do j=1,jmax
            do i=1,imax
              depeh(i,j,k)=depeh(i,j,k)+depe(i,j,k)
              depeh2(i,j,k)=depeh2(i,j,k)+depe(i,j,k)*depe(i,j,k)
              depe(i,j,k)=0.D0
            end do
          end do
        end do

        faexps=faexps+faexp
        faexp2s=faexp2s+faexp*faexp
        faexp=0.0
        fexpss=fexpss+fexps
        fexps2s=fexps2s+fexps*fexps
        fexps=0.0

        ncount = ncount + 1         ! Count total number of actual cases

!                          ======================
        if (iwatch .gt. 0) call swatch(-1,iwatch)
!                          ======================

                                               ! -----------------------
      end do                                   ! End of CALL SHOWER loop
                                               ! -----------------------

      tt=etime(tarray)
      tt1=tarray(1)
      cputime=tt1-tt0
      write(2,250) cputime
250   format(' Elapsed Time (sec)=',G15.5)

!                        =======================
      if (iwatch .gt. 0) call swatch(-88,iwatch)
!                        =======================

!-----------------------------------------------------------------------
! Step 9:  Output-of-results
!-----------------------------------------------------------------------

!     ---------------------
!     Write out the results
!     ---------------------
      write(6,260) ncount,ncases,totke
260   FORMAT(/,' Ncount=',I10,' (actual cases run)',/,
     *       ' Ncases=',I10,' (number of cases requested)',/,
     *       ' TotKE =',G15.5,' (total KE (MeV) in run)')

      if (totke .le. 0.D0) then
        write(6,270) totke,availke,ncount
270     FORMAT(/,' Stopped in MAIN with TotKE=',G15.5,/,
     *         ' AvailKE=',G15.5, /,' Ncount=',I10)
        stop
      end if

!-----------------------------
!     Sampled source spectrum
!-----------------------------

      write(1,280) sposi 
280   FORMAT(/' Absorbed energy inside phantom for 1.253MeV photon'/
     *' Source position ',F10.1,' cm from phantom surface'/ 
     *' Within 1cm x 1 cm area after 5 cm air')

      write(1,290) ncases, xhbeam, yhbeam                            
290   FORMAT(1X,I8,' photons normally incident from front side'/
     *' Half width of beam is ',G15.5,'cm for X and ',G15.5,'cm for Y')

!      ---------------------------------------
!      Calculate average and its uncertainties
!      ---------------------------------------

        do k=1,kmax
          do j=1,jmax
            do i=1,imax
              irl=1+i+(j-1)*imax+(k-1)*ijmax
              amass=(xbound(i+1)-xbound(i))*
     *              (ybound(j+1)-ybound(j))*
     *              (zbound(k+1)-zbound(k))*rhor(irl)
              dose(i,j,k)=depeh(i,j,k)/ncases
              depeh2(i,j,k)=depeh2(i,j,k)/ncases
              doseun(i,j,k)=dsqrt((depeh2(i,j,k)-
     *                      dose(i,j,k)*dose(i,j,k))/ncases)
              dose(i,j,k)=dose(i,j,k)*1.602D-10/amass
              doseun(i,j,k)=doseun(i,j,k)*1.602D-10/amass
            end do
          end do
        end do
        
!----------------------------------------------
!     Print out the results of central phantom
!----------------------------------------------
      i=imax/2+1
      j=jmax/2+1
      do kkk=2,kmax-1 
        depths=zbound(kkk)
        depthl=zbound(kkk+1)
        irl=1+i+(j-1)*imax+(kkk-1)*ijmax
        write(6,300) depths,depthl,(media(ii,med(irl)),ii=1,24),
     *  rhor(irl),dose(i,j,kkk),doseun(i,j,kkk)
300     FORMAT(' At ',F4.1,'--',F4.1,'cm (',24A1,',rho:',F8.4,')=',
     *  G13.5,'+-',G13.5,'Gy/incident')
      end do

!-------------------------------------------------
!     Calculate average exposure and its deviation
!-------------------------------------------------
      area=(xbound(i+1)-xbound(i))*(ybound(j+1)-ybound(j))
      faexpa=faexps/ncases
      faexp2s=faexp2s/ncases
      faexrr=dsqrt((faexp2s-faexpa*faexpa)/ncases) 
      faexpa=faexpa*1.6E-10/area
      faexrr=faexrr*1.6E-10/area
      fexpsa=fexpss/ncases
      fexps2s=fexps2s/ncases
      fexerr=dsqrt((fexps2s-fexpsa*fexpsa)/ncases) 
      fexpsa=fexpsa*1.6E-10/area
      fexerr=fexerr*1.6E-10/area
      if (faexpa.gt.0.0) then
        bsfa=fexpsa/faexpa
        bsferr=bsfa*dsqrt((faexrr/faexpa)**2.+(fexerr/fexpsa)**2.)
        write(6,310) faexpa,faexrr,fexpsa,fexerr,bsfa,bsferr
310     FORMAT(/' Exposure in free air (using mu_en) =', G15.5,'+-',G15.
     *  5,' Gy/incident'/ ' Exposure at phantom surface (using mu_en) ='
     *  , G15.5,'+-',G15.5,'Gy/incident'/ ' Backscattering factor =',G15
     *  .5,'+-',G15.5)
      else
        write(6,320) faexpa,faexrr,fexpsa,fexerr
320     FORMAT(/' Exposure in free air (using mu_en) =', G15.5,'+-',G15.
     *  5,' Gy/incident'/ ' Exposure at phantom surface (using mu_en) ='
     *  , G15.5,'+-',G15.5,'Gy/incident')
      end if

!     ---------------------------
!     Write out the whole results
!     ---------------------------
      do idose=1,idgrp         ! Loop over groups of regions to analyze
        if (izscan(idose).ne.0) then   ! Do output with one Z scan per page
!         Number of sets of depth per page
          k = (kdosu(idose) - kdosl(idose))
          k = k + k/5 + 7
          nperpg = 60/k
          write(6,330) Title
330       FORMAT(/10X,80A1//T10,'xyz(V01) dose outputs  Gy.cm**2',
     *    '(or Gy/incident particle for 0 area)')
          ipage=1   ! Count how many zgroups printed on this page

          do i=idosl(idose),idosu(idose)
            do j=jdosl(idose),jdosu(idose),4
              jl=j
              ju=min(j+3,jdosu(idose))
              write(6,340) xbound(i),xbound(i+1),i
340           FORMAT(/T15,'For x=',F10.3,' to',F10.3,5X,'i=',I3)
              write(6,350) (ybound(jj),jj=jl,ju+1)
350           FORMAT(' ybounds:',F7.3,F12.3,3F17.3)
              write(6,360)(jj,jj=jl ,ju)
360           FORMAT(T10,'j=',t17,5(I4,13X))
              write(6,370) zbound(kdosl(idose))
370           FORMAT(' zbounds (',F10.3,')')
              do k=kdosl(idose),kdosu(idose)
                write(6,380) zbound(k+1),k,(dose(i,jj,k),
     *          min(99.9, 100.*doseun(i,jj,k)/dose(i,jj,k)),
     *          jj=jl,ju)
380             FORMAT(F8.3,I4,4(1PE11.3,'-',0PF4.1,'%') )
                if (mod(k,5).eq.0) then
                  write(6,390)
390               FORMAT(' ')
                end if
              end do

              if(mod(ipage,nperpg).eq.0.and.(ju.ne.jdosu(idose).
     *          or.i.ne.idosu(idose))) then
                write(6,330) Title
                ipage=1
              else
                ipage=ipage+1
              end if
            end do   ! end j-loop
          end do     ! end i-loop

        else                   ! Output x scans each page
          i=idosu(idose)-idosl(idose)
          i=i+i/5+7
          nperpg=60/i        ! Number of sets of bins per page
          write(6,330) Title
          ipage=1
          
          do k=kdosl(idose),kdosu(idose)
            do j=jdosl(idose),jdosu(idose),4
              jl=j
              ju=min(j+3,jdosu(idose))
              write(6,400) zbound(k),zbound(k+1),k
400           FORMAT(/T15,'for z=',F10.3,' to',F10.3,5X,'k=',I3)
              write(6,410) (ybound(jj),jj=jl,ju+1)
410           FORMAT(' Ybounds:',F7.3,F12.3,3F17.3)
              write(6,420) (jj, jj=jl,ju)
420           FORMAT(T10,'j=',T17,5(I4,13X))
              write(6,430) xbound(idosl(idose))
430           FORMAT(' Xbounds (',F10.3,')')

              do i=idosl(idose),idosu(idose)
                write(6,440) xbound(i+1),i,(dose(i,jj,k),
     *          min(99.9, 100.*doseun(i,jj,k)/dose(i,jj,k)),
     *          jj=jl,ju)
440             FORMAT(F8.3,I4,4(1PE11.3,'-',0PF4.1,'%') )
                if (mod(i,5).eq.0) then
                  write(6,fmt="(' ')")
                end if
              end do
               
              if(mod(ipage,nperpg).eq.0.and.(ju.ne.jdosu(idose).
     *          or.k.ne.kdosu(idose))) then
                write(6,330) Title
                ipage=1
              else 
                ipage=ipage+1
              end if
            end do                ! end j-loop
          end do                  ! end k-loop
        end if                    ! end of x scan per page output
      end do                    ! end of idose loop

!     =============================
      call ecnsv1(1,nreg,totke)
!     =============================

!     ====================
      call counters_out(1)
!     ====================

      stop

      end

!-------------------------last line of main code------------------------
!----------------------------getvoxel.f---------------------------------
! Version:   060907-1300
!         Read material and geomery data from egs5job.inp
!         121119-1300     Fixed degug errors
!-----------------------------------------------------------------------
!23456789|123456789|123456789|123456789|123456789|123456789|123456789|12
! ----------------------------------------------------------------------
! Auxiliary subroutine for use with the EGS5 Code System
! ----------------------------------------------------------------------
! This is a data-entry subprogram for use with a general-purpose
! egs5 user code to do cartesian coordinate dose deposition studies. 
! Every voxel (volume element) can have different materials and/or 
! varying densities (for use with CT data).
! Basic pats of this subroutine related with geometry taken from 
! xyzdos.mor.
! ----------------------------------------------------------------------
!
!   voxels are labeled by indicies (i,j,k) and defined by: 
!        xbound(i) <= x < xbound(i+1)     i <= imax 
!        ybound(j) <= y < ybound(j+1)     j <= jmax 
!        zbound(k) <= z < zbiund(k+1)     k <= kmax 
!
! -------------------
! SUBROUTINE ARGUMENT
! -------------------
!  nreg      Number of regions in geometry (determined by data input).
!
! ----------------
! UNIT ASSIGNMENTS
! ----------------
!  Unit 4       Input file.
!  Unit ifto    Output file.
!
! ----------
! INPUT FILE
! ----------
!  Record 1  title (80A1)        Title line.
!  --------
!  Record 2  nmed (I10)          Number of media in problem.
!  --------
!  Record 3  media(j,i) (24A1)   Media names (j=1,24, I=1,nmed lines).
!  --------                     Note that entire volume is initially 
!                                set to medium.
! ---------
!  Record 4  chard               characteristic distance for each medium
! ---------
!  --------
!  Record 5   maxx, maxy, maxz   Number of voxels in the X,Y,Z directions
!  --------                      If <0, it means that number of equally 
!                                spaced boundaries will be input.
!  --------
!  Record 6   xbound
!  --------  
!                    i.e. repeat the following replacing (i and x) by
!                      (j and y) and (k and z) respectively.
!    if maxx > 0
!               input, one per line, the maxx + 1  x boundaries
!
!    if maxx < 0
!               input smallest x boundary, followed by  abs(maxx) pairs
!               one per line: voxel width, # voxels with this width
!
!     for example: starting at record 5
!              -1,-1,-1
!              0.0
!              1.0,16
!              0.0
!              1.0,16
!              0.0
!              1.0,16
!     defines a 16x16x16 cube of 1cm**3 voxels with a total of 4097 reg
!    or
!              -1,-1,3
!              0.0
!              1.0,16
!              0.0
!              1.0,16
!              0.0
!              5.0
!              10.0
!      defines a 16x16x10 cube with 1x1x5 cm voxels stacked 2 deep
!  --------
!  Record 7    ybound
!  --------
!  Record 8    zbound
!  -------- 
!  ---------
!  Record 9 ecutin,pcutin,       For medium 1 at first
!  ---------ipeangsw,            Switches for PE-angle sampling, 
!           iedgeflsw,           K & L-edge fluorescence, 
!           iaugersw,            K & L-Auger
!           iraysw,              Rayleigh scattering,
!           ipolarsw,            Linearly-polarized photon scattering,
!           incohrsw,            S/Z rejection,
!           iprofrsw,            Doppler broadening,
!           impacrsw             electron impact ionization (0=off, 1=on).
!  --------
! Record 10 il,iu, jl,ju, kl,ku, medtmp, rhotmp,ecutin,pcutin
!  --------                    Line is repeated until a blank line found
!                              All regions default to medium 1 with its 
!                              default density unless changed here.
!                              For all voxels with
!                               IL <= I <= IU
!                               JL <= J <= JU
!                               KL <= K <= KU
!                       the medium used is MEDIUM and the density used is
!                       DENSITY. If DENSITY=0.0, the default value for that
!                       medium is used (faster than entering default density
!                       here).
!                       If iu and il are zero, it means the end of define.
!                                If medium not 0, following option is set
!                                to the regions above.
! ---------
! Record 10a ipeangsw,            Switches for PE-angle sampling, 
! --------- iedgeflsw,           K & L-edge fluorescence, 
!           iaugersw,            K & L-Auger
!           iraysw,              Rayleigh scattering,
!           ipolarsw,            Linearly-polarized photon scattering,
!           incohrsw,            S/Z rejection,
!           iprofrsw,            Doppler broadening,
!           impacrsw             electron impact ionization (0=off, 1=on).
!
! --------
! Record 11  il,iu, jl,ju, kl,ku,izscan
! ---------                      Regions for which the dose will be output.
!                                IZSCAN non-zero to get z-scan per page, 
!                                otherwise output is an x-scan per page.
!
! ---------
! Record 12 iwatch               Switch for tracking events with swatch:
! ---------                        (0=No, 1=each interaction,
!                                   2=each step)
! ---------
! Record 13 ibrdst,iprdst,       Switches for bremsstrahlung and pair
! --------- ibrspl,nbrspl        production ANGLE SAMPLING, and brems-
!                                strahlung SPLITTING:
!                                ibrdst=0 No (use default: theta=m/E)
!                                       1 Yes (recommended)
!                                iprdst=0 No (use default: theta=m/E)
!                                       1 Yes (low-order distribution)
!                                       2 Yes (recommended)
!                                ibrspl=0 No
!                                       1 Yes (NBRSPL=splitting factor)
! ----------------------------------------------------------------------

      subroutine getvoxel(ifto)

      implicit none

      include 'include/egs5_h.f'                ! Main EGS "header" file

      include 'include/egs5_bounds.f'    ! COMMONs required by EGS5 code
      include 'include/egs5_brempr.f'
      include 'include/egs5_edge.f'
      include 'include/egs5_eiicom.f'
      include 'include/egs5_elecin.f'
      include 'include/egs5_media.f'
      include 'include/egs5_misc.f'
      include 'include/egs5_thresh.f'
      include 'include/egs5_uphiot.f'
      include 'include/egs5_useful.f'
      include 'include/egs5_userpr.f'
      include 'include/egs5_usersc.f'
      include 'include/egs5_uservr.f'
      include 'include/egs5_userxt.f'

      include 'pegscommons/mscom.f'                        ! PEGS common

      include 'auxcommons/aux_h.f'   ! Auxiliary-code "header" file

      include 'auxcommons/edata.f'        ! Auxiliary-code COMMONs
      include 'auxcommons/geoxyz.f'
      include 'auxcommons/instuf.f'
      include 'auxcommons/voxel.f'
      include 'auxcommons/watch.f'

      include 'include/randomm.f'         ! Additional (non-EGS5) COMMON

      integer ifto                         ! Argument

      real*8                                           ! Local variables
     * ecutin,ecutmn,ek0,pcutin,rhotmp,totphi,
     * thetax,thetay,thetaz,xlower,
     * xupper,ylower,yupper,width

      integer i,igroup,ii,iiz,il,in,irl,iu,ixinu,
     * ixx,izn,j,jl,ju,jxx,jyinu,k,kl,ku,maxbd,maxx,maxy,
     * maxz,medtmp,moreOutput,n,ner,ngroup,nn,nnn

      integer ipeangsw,iedgeflsw,iaugersw,iraysw,ipolarsw,incohrsw,
     *        iprofrsw,impacrsw

      data moreOutput/0/       ! Change this from 0 to 1 for more output

      write(ifto,100)
100   FORMAT(//,T25,'+----------------------------------------+',
     *        /,T25,'| EGS5 User Code using subroutine voxel  |',
     *        /,T25,'+----------------------------------------+',
     *        /,T25,'|  NOTE:  X-Y-Z Voxel geometry.          |',
     *        /,T25,'|         X-Y plane on the page          |',
     *        /,T25,'|         (X to the right, Y upwards,    |',
     *        /,T25,'|           Z out).                      |',
     *        /,T25,'+----------------------------------------+',
     *        //)

!  SJW 02-May-2002  New subroutine calls to initialize data no
!  longer set in block data because of size issues

!     ==============
      call block_set                 ! Initialize some general variables
!     ==============

! ---------------
! Record 1: title
! ---------------
      read(4,110) title
 110  FORMAT(80A1)
      write(ifto,120) title
 120  FORMAT(8x,71('-')/'$TITLE: '/'+',3X,80A1/8X,71('-'))

! --------------
! Record 2: nmed
! --------------
      read(4,*) nmed
      if (nmed.eq.0 .or. nmed .gt. MXMED) then
        write(ifto,130) nmed
 130    FORMAT(' *** Stopped in Getvoxel with nmed=',I5,' > MXMED')
        stop
      end if
      write(ifto,140) nmed
 140  FORMAT(' Number of media : ',I5,/)

! ---------------
! Record 3: media
! ---------------
      do i=1,nmed
        read(4,150) (media(j,i),j=1,24)
 150    FORMAT(24A1)
        write(ifto,160) i,(media(j,i),j=1,24)
 160    FORMAT(' MEDIUM=',I5,' ==> ',24A1)
      end do

!     charD is defined to each medium.
! ----------------
! Record 4: chard
! ----------------
      read(4,*) (chard(i),i=1,nmed)

      write(ifto,*) 'chard =',(chard(i),i=1,nmed)

! ---------------------------
! Record 5: maxx, maxy, maxz
! ---------------------------
      read(4,*) maxx,maxy,maxz

!     Check bin-number.
      if (maxx.eq.0) maxx =1
      if (maxx.gt.LIMAX) then
        write(ifto,'(A,I4,A,I4/A)') ' maxx(',maxx,
     *     ') in record 5 is larger than',LIMAX,
     *     ' You must increase LIMAX in auxcommon/aux_h.f '
        stop
      end if
      if (maxy.eq.0) maxy =1
      if (maxy.gt.LJMAX) then
        write(ifto,'(A,I4,A,I4/A)') ' maxy(',maxy,
     *     ') in record 5 is larger than',LJMAX,
     *     ' You must increase LJMAX in auxcommon/aux_h.f '
        stop
      end if
      if (maxz.eq.0) maxz =1
      if (maxz.gt.LKMAX) then
        write(ifto,'(A,I4,A,I4/A)') ' maxz(',maxz,
     *     ') as bin number is larger than',LKMAX,
     *     ' You must increase LKMAX in auxcommon/aux_h.f '
        stop
      end if

      write(ifto,170) maxx,maxy,maxz
170   FORMAT ('+',3I6)
 
      maxbd=LIMAX
      write(ifto,180)
180   FORMAT(/T20,'Input boundaries in the x direction')

! -----------------
! Record 6   xbound 
! -----------------
      if (maxx.gt.0) then            ! Just pick up boundaries.
        do i=1,maxx
          write(ifto,190) i
190       FORMAT(' Small boundary for region(',I3,')  ')
          read(4,*) xbound(i)
          if(i.ne.1) then
            if (xbound(i).le.xbound(i-1)) write(ifto,200)
200         FORMAT(' Boundary out of order**************')
          end if
          write(ifto,210) xbound(i)
210       FORMAT('+',T10,F12.3)
        end do
        write(ifto,220) maxx
220     FORMAT(' Outer boundary for region(',I3,')  ')
        read(4,*) xbound(maxx+1)
        write(ifto,230) xbound(maxx+1)
230     FORMAT('+',T10,F12.3)
      else                           ! Input groups of region.
        write(ifto,240)
240     FORMAT(' Initial boundary: ')
        read(4,*) xbound(1)
        write(ifto,250) xbound(1)
250     FORMAT('+',F12.3)
        ngroup=-maxx
        maxx=0
        do igroup=1,ngroup
          write(ifto,260)
260       FORMAT(' Width in this group, no. of regions in group: ')
          read(4,*) width,nn
          if(nn.le.0) nn=1
          if(width.le.0.0) width=1.D0
          write(ifto,270) width,nn
270       FORMAT('+',F12.3,I5)
          nnn=min(nn,maxbd-maxx)
          if (nnn.ne.0) then
            do in=maxx+1,maxx+nnn
              xbound(in+1)=xbound(in)+width
            end do
          end if
          if (nn.ne.nnn) then
            write(ifto,280)
280         FORMAT(T15,'*** No. of X-direction reduced ***'/)
          end if
          maxx=maxx+nnn
        end do
        if (maxx.gt.LIMAX) then
          write(ifto,'(A,I4,A,I4/A)') ' maxx(',maxx,
     *     ') calculated in maxx < 0 mode is larger than',LIMAX,
     *     ' You must increase LIMAX in auxcommon/aux_h.f '
          stop
        end if
        write(ifto,290) (xbound(i),i=1,maxx+1)
290     FORMAT(' Boundaries'/(6F12.3))
      end if

      imax=maxx

      maxbd=LJMAX
      write(ifto,300)
300   FORMAT(/T20,'Input boundaries in the y direction')

! -----------------
! Record 7   ybound 
! -----------------
      if (maxy.gt.0) then            ! Just pick up boundaries.
        do i=1,maxy
          write(ifto,190) i
          read(4,*) ybound(i)
          if(i.ne.1) then
            if (ybound(i).le.ybound(i-1)) write(ifto,200)
          end if
          write(ifto,210) ybound(i)
        end do
        write(ifto,220) maxy
        read(4,*) ybound(maxy+1)
        write(ifto,230) ybound(maxy+1)
      else                           ! Input groups of region.
        write(ifto,240)
        read(4,*) ybound(1)
        write(ifto,250) ybound(1)
        ngroup=-maxy
        maxy=0
        do igroup=1,ngroup
          write(ifto,260)
          read(4,*) width,nn
          if(nn.le.0) nn=1
          if(width.le.0.0) width=1.D0
          write(ifto,270) width,nn
          nnn=min(nn,maxbd-maxy)
          if (nnn.ne.0) then
            do in=maxy+1,maxy+nnn
              ybound(in+1)=ybound(in)+width
            end do
          end if
          if(nn.ne.nnn) then
            write(ifto,280)
          end if
          maxy=maxy+nnn
        end do
        if (maxy.gt.LJMAX) then
          write(ifto,'(A,I4,A,I4/A)') ' maxy(',maxy,
     *     ') calculated in maxy < 0 mode is larger than',LJMAX,
     *     ' You must increase LJMAX in auxcommon/aux_h.f '
          stop
        end if
        write(ifto,290) (ybound(i),i=1,maxy+1)
      end if

      jmax=maxy

      maxbd=LKMAX
      write(ifto,310)
310   FORMAT(/T20,'Input boundaries in the z direction')

! -----------------
! Record 8   zbound 
! -----------------
      if (maxz.gt.0) then            ! Just pick up boundaries.
        do i=1,maxz
          write(ifto,190) i
          read(4,*) zbound(i)
          if(i.ne.1) then
            if (zbound(i).le.zbound(i-1)) write(ifto,200)
          end if
          write(ifto,210) zbound(i)
        end do
        write(ifto,220) maxz
        read(4,*) zbound(maxz+1)
        write(ifto,230) zbound(maxz+1)
      else                           ! Input groups of region.
        write(ifto,240)
        read(4,*) zbound(1)
        write(ifto,250) zbound(1)
        ngroup=-maxz
        maxz=0
        do igroup=1,ngroup
          write(ifto,260)
          read(4,*) width,nn
          if(nn.le.0) nn=1
          if(width.le.0.0) width=1.D0
          write(ifto,270) width,nn
          nnn=min(nn,maxbd-maxz)
          if (nnn.ne.0) then
            do in=maxz+1,maxz+nnn
              zbound(in+1)=zbound(in)+width
            end do
          end if
          if(nn.ne.nnn) then
            write(ifto,280)
          end if
          maxz=maxz+nnn
        end do
        if (maxz.gt.LKMAX) then
          write(ifto,'(A,I4,A,I4/A)') ' maxz(',maxz,
     *     ') calculated in maxz < 0 mode is larger than',LKMAX,
     *     ' You must increase LKMAX in auxcommon/aux_h.f '
          stop
        end if
        write(ifto,290) (zbound(i),i=1,maxz+1)
      end if

      kmax=maxz

      ijmax = imax*jmax
      irmax = 1 + ijmax*kmax
      nreg = irmax
      
      write(ifto,320) imax,jmax,kmax,nreg
320   FORMAT(' imax, jmxa, kmax, nreg =',4I8)

!     Check nreg
      if (nreg .gt. MXREG) then
        write(ifto,330) nreg
330     FORMAT(' *** Stopped in getvoxel with nreg=',I5,' > MXREG')
        stop
      end if
      write(ifto,340) nreg
340   FORMAT(/,' number of region (nreg) =',I5,/,
     *         ' nreg includes outside vacuum region (regin=1)')
      
!     Set all region except 1 set to medium=1.
      med(1)=0

! ----------------------------------------------------------
! Record 9: ipeangsw,iedgeflsw,iaugersw,iraysw,
!           ipolarsw,incohrsw,iprofrsw,impacrsw for medium 1
! ----------------------------------------------------------
        read(4,*) ecutin,pcutin,ipeangsw,iedgeflsw,iaugersw,iraysw,
     *            ipolarsw,incohrsw,iprofrsw,impacrsw

        write(ifto,350) ecutin,pcutin,ipeangsw,iedgeflsw,iaugersw,
     *             iraysw,ipolarsw,incohrsw,iprofrsw,impacrsw
350     FORMAT(/' Medium 1'/' ecut =',G15.5,' and pcut =',G15.5/
     *' ipeangsw=',I3,',iedgeflsw=',I3,',iaugersw=',I3,'iraysw=',I3/
     *' ipolarsw=',I3,' ,incohrsw=',I3,',iprofrsw=',I3,',impacrsw=',I3)

      do i=2,irmax
        med(i)=1
        if (pcutin .gt. 0.)  pcut(i) = pcutin
        if (ecutin .gt. 0.)  ecut(i) = ecutin + RM
        iphter(i) = ipeangsw
        iedgfl(i) = iedgeflsw
        iauger(i) = iaugersw
        iraylr(i) = iraysw
        lpolar(i) = ipolarsw
        incohr(i) = incohrsw
        iprofr(i) = iprofrsw
        impacr(i) = impacrsw
      end do

!  -------- 
!  Record 10 il,iu, jl,ju, kl,ku, medtmp, rhotmp, ecutin, pcutin
!  -------- (7I5,3F10.0)       Line is repeated until a blank line found

360   write(ifto,370)
370   FORMAT(' Lower,upper i, j, k, medium, density')

      read(4,*) il,iu,jl,ju,kl,ku,medtmp,rhotmp,ecutin,pcutin
      if(il.eq.0 .and. iu.eq.0) go to 400

!     Check il etc.
      if(il.lt.0) il=1
      if(iu.lt.0 .or. iu.ge.imax) iu=imax
      if(jl.le.0) jl=1
      if(ju.le.0 .or. ju.ge.jmax) ju=jmax
      if(kl.le.0) kl=1
      if(ku.le.0 .or. ku.ge.kmax) ku=kmax
      
!     Check medtmp
      if(medtmp.lt.0 .or. medtmp.gt.nmed) medtmp=1
      
      write(ifto,380) il,iu,jl,ju,kl,ku,medtmp,rhotmp
380   FORMAT('+',3('(',I3,I4,')'), I4, F10.3)

      if (medtmp.ne.0) then
! ---------------------------------------------------
! Record 10a: ipeangsw,iedgeflsw,iaugersw,iraysw,
!             ipolarsw,incohrsw,iprofrsw,impacrsw
! ---------------------------------------------------
        read(4,*) ipeangsw,iedgeflsw,iaugersw,iraysw,ipolarsw,
     *            incohrsw,iprofrsw,impacrsw

        write(ifto,390) ecutin,pcutin,ipeangsw,iedgeflsw,iaugersw,
     *             iraysw,ipolarsw,incohrsw,iprofrsw,impacrsw
390     FORMAT(' ecut =',G15.5,' and pcut =',G15.5/
     *' ipeangsw=',I3,',iedgeflsw=',I3,',iaugersw=',I3,'iraysw=',I3/
     *' ipolarsw=',I3,' ,incohrsw=',I3,',iprofrsw=',I3,',impacrsw=',I3)
     
        do i=il,iu
          do j=jl,ju
            do k=kl,ku
              irl=1+i+(j-1)*imax+(k-1)*ijmax
              med(irl)=medtmp
              if (rhotmp.ne.0) rhor(irl)=rhotmp
              if (pcutin .gt. 0.)  pcut(irl) = pcutin
              if (ecutin .gt. 0.)  ecut(irl) = ecutin+ RM
              iphter(irl) = ipeangsw
              iedgfl(irl) = iedgeflsw
              iauger(irl) = iaugersw
              iraylr(irl) = iraysw
              lpolar(irl) = ipolarsw
              incohr(irl) = incohrsw
              iprofr(irl) = iprofrsw
              impacr(irl) = impacrsw
            end do
          end do
        end do
      else  
        do i=il,iu
          do j=jl,ju
            do k=kl,ku
              irl=1+i+(j-1)*imax+(k-1)*ijmax
              med(irl)=0
            end do
          end do
        end do
      end if

      go to 360 

400   continue
! --------
! Record 11  il,iu, jl,ju, kl,ku,izscan
! ---------
      write(ifto,410)
410   FORMAT(' 3 pairs defining lower,upper x,y,z indecses of dose',
     * 'regions'/' for which results are to be output'/
     * ' izscan non-zero : scan per page'/
     * ' One set of 6 per line, end with all zero')

      idgrp=0
420   idgrp=idgrp+1
      write(ifto,430)
430   FORMAT('$: ')
      read(4,*) idosl(idgrp),idosu(idgrp),jdosl(idgrp),jdosu(idgrp),
     *          kdosl(idgrp),kdosu(idgrp),izscan(idgrp)

      if(idosl(idgrp).eq.0 .and. idosu(idgrp).eq.0) go to 460  ! End of define.
      
      if(idosl(idgrp).le.0) idosl(idgrp)=1
      if(idosu(idgrp).le.0 .or. idosu(idgrp).ge.imax) idosu(idgrp)=imax
      if(jdosl(idgrp).le.0) jdosl(idgrp)=1
      if(jdosu(idgrp).le.0 .or. jdosu(idgrp).ge.jmax) jdosu(idgrp)=jmax
      if(kdosl(idgrp).le.0) kdosl(idgrp)=1
      if(kdosu(idgrp).le.0 .or. kdosu(idgrp).ge.kmax) kdosu(idgrp)=kmax
       
      write(ifto,450) idosl(idgrp),idosu(idgrp),jdosl(idgrp),
     *            jdosu(idgrp),kdosl(idgrp),kdosu(idgrp),izscan(idgrp)
450   FORMAT('+',T5,3(I6,I4),I6)

      go to 420

460   continue

      idgrp=idgrp-1
      
      if(idgrp.gt.LMXDOS) then
        write(ifto,470) idgrp,LMXDOS
470     FORMAT(' idgrp(=',I5,') must be less than LMXDOS(=',I5,')'/ 
     *         ' Or you must chnage LMXDOS in xyzdose_h.f')
      end if
      
! -----------------
! Record 12: iwatch
! ----------------- 
      read(4,*) iwatch

      write(ifto,790) iwatch
790   FORMAT(//,' SWATCH tracking switch: iwatch=',I2,
     *          ' (0=off, 1=each interaction, 2=each step)')

! --------------------------------------
! Record 13: ibrdst,iprdst,ibrspl,nbrspl
! --------------------------------------
      read(4,*) ibrdst,iprdst,ibrspl,nbrspl
      write(ifto,800) ibrdst,iprdst,ibrspl,nbrspl
800   FORMAT(/,' IBRDST=',I2,/,' IPRDST=',I2,/,' IBRSPL=',I2,' (NBRSPL='
     *,I5,')')

      if (ibrspl .gt. 0) then
        if (nbrspl .gt. 0) then
          fbrspl = 1.0/float(nbrspl)
        else
          write(ifto,810) ibrspl,nbrspl
810       FORMAT(//,' Stopped in Getvoxel with IBRSPL=',
     *          I5,' and NBRSPL=',I5)
          stop
        end if
      end if

                                                        ! --------------
      return                                            ! Return to MAIN
                                                        ! --------------
      end

!-------------------------last line of getvoxel.f------------------------
!-------------------------------ausgab.f--------------------------------
! Version:   031108-1300
!            060801-1000
! Reference: SLAC-265 (p.19-20, Appendix 2)
!-----------------------------------------------------------------------
!23456789|123456789|123456789|123456789|123456789|123456789|123456789|12

! ----------------------------------------------------------------------
! Required subroutine for use with the EGS5 Code System
! ----------------------------------------------------------------------
! A simple AUSGAB to:
!
!   1) Score energy deposition
!   2) Print out stack information
!   3) Print out particle transport information (if switch is turned on)
!

! ----------------------------------------------------------------------

      subroutine ausgab(iarg)

      implicit none

      include 'include/egs5_h.f'                ! Main EGS "header" file

      include 'include/egs5_epcont.f'    ! COMMONs required by EGS5 code
      include 'include/egs5_stack.f'

      include 'auxcommons/aux_h.f'   ! Auxiliary-code "header" file

      include 'auxcommons/etaly1.f'        ! Auxiliary-code COMMONs
      include 'auxcommons/geoxyz.f'
      include 'auxcommons/lines.f'
      include 'auxcommons/ntaly1.f'
      include 'auxcommons/voxel.f'
      include 'auxcommons/watch.f'

                                         ! Variables to score
      common/score/                             ! Variables to score
     *       depe(LIMAX,LJMAX,LKMAX),faexp,fexps,maxpict
      real*8 depe,faexp,fexps
      integer maxpict

      integer                                                ! Arguments
     * iarg

      real*8                                           ! Local variables
     * cmod,dcon,edepwt,encoea,esing

      integer i,irl,irx,iry,irz,iql,j,k

!     -----------------------------------------------------------------
!     Print out particle transport information (if switch is turned on)
!     -----------------------------------------------------------------
!                        ========================
      if (iwatch .gt. 0) call swatch(iarg,iwatch)
!                        ========================

!  ---------------------------------
!  Keep track of how deep stack gets
!  ---------------------------------
        if (np.gt.MXSTACK) then
          write(6,100) np,MXSTACK
100       FORMAT(//' In AUSGAB, np=',I3,' >= maximum stack',
     *         ' allowed which is',I3/1X,79('*')//)
          stop
        end if
!     ------------------------
!     Set some local variables
!     ------------------------
      irl = ir(np)
      iql = iq(np)
      edepwt = edep*wt(np)

!     ----------------------------------------------------------------
!     Print out stack information (for limited number cases and lines)
!     ----------------------------------------------------------------
      if (ncount .le. nwrite .and. ilines .le. nlines) then
        ilines = ilines + 1
        write(6,101) e(np),x(np),y(np),z(np),u(np),v(np),w(np),
     *               iql,irl,iarg
 101    FORMAT(7G15.7,3I5)
      end if

!     -----------------------------------------------------------
!     Keep track of energy deposition (for conservation purposes)
!     -----------------------------------------------------------
      if (iarg .gt. 5) return

      esum(iql+2,irl,iarg+1) = esum(iql+2,irl,iarg+1) + edepwt
      nsum(iql+2,irl,iarg+1) = nsum(iql+2,irl,iarg+1) + 1

      i=mod(irl-1,imax)
      if (i.eq.0) i=imax
      k=1+(irl-1-i)/ijmax
      j=1+(irl-1-i-(k-1)*ijmax)/imax

      if (irl.gt.1.and.edep.ne.0.D0) then
        depe(i,j,k)=depe(i,j,k)+edepwt
      end if

!--------------------------------
!     Check cross phantom surface
!--------------------------------
      if(i.eq.imax/2+1.and.j.eq.jmax/2+1) then  ! X-Y central region
        if (abs(irl-irold).eq.ijmax.and.iq(np).eq.0) then
          if ((w(np).gt.0.0.and.k.eq.2).or.
     *     (w(np).le.0.0.and.k.eq.1)) then
            if (dabs(w(np)).ge.0.0349) then
              cmod=dabs(w(np))
            else
              cmod=0.01745
            end if
            esing=e(np)
            dcon=encoea(esing)            ! PHOTX data
            fexps=fexps+e(np)*dcon*wt(np)/cmod
            if (w(np).lt.0.0) latch(np)=1
            if (w(np).gt.0.0.and.latch(np).eq.0) then
              faexp=faexp+e(np)*dcon*wt(np)/cmod
            end if
          end if
        end if
      end if

!     ------------------------------------
!     Output particle information for plot
!     ------------------------------------
      if (ncount.le.maxpict) then
        call plotxyz(iarg,np,iq(np),x(np),y(np),z(np),e(np),ir(np),
     *       wt(np),time(np))
      end if

      return

      end

!--------------------------last line of ausgab.f------------------------
!-------------------------------howfar.f--------------------------------
! Version:   030831-1300
! Reference: SLAC-265 (p.19-20, Appendix 2)
!-----------------------------------------------------------------------
!23456789|123456789|123456789|123456789|123456789|123456789|123456789|12

! ----------------------------------------------------------------------
! Required (geometry) subroutine for use with the EGS5 Code System
! ----------------------------------------------------------------------
!   HOWFAR routine to use with a generalized cartesian coordinate system
!   for voxel geometry.
!
!   Geometrical information is passed in common/geoxyz
!    xbound(MXXPLNS+1),ybound(MXYPLNS+1),zbound(MXZPLNS+1),imax,jmax,
!    kmax,ijmax,irmax
!     xbound etc are the X, Y and Z boundaries defining the voxels
!     MXXPLNS etc are the maximum number of planes in each direction
!           as defined in the auxiliary-code header file.
!     imax etc are the actual number of elements in each direction for
!           this particular calculation
!     ijmax = imax*jmax   a useful number
!     irmax = 1 + ijmax*kmax  the total number of regions in the
!           current problem
!
!     Each voxel is defined by a triple of integers (i,j,j) (but called
!           irx,iry and irz in this routine) such that:
!
!           xbound(i) <= x < xbound(i+1)     1 < i < imax
!           ybound(j) <= y < ybound(j+1)     1 < j < jmax
!           zbound(k) <= z < zbound(k+1)     1 < k < kmax
!
!  The X axis is up the page, the Y axis to the right and Z into the page
!
!  The region number is defined as:
!       ir = 1 + i + (j-1)*imax + (k-1)*ijmax
!
!  The routine sets DNEAR  Note that in problems where the typical
!  step size is of the order of the region dimensions, then computing
!  DNEAR can decrease efficiency. In this case the two lines containing
!  DNEAR should be commented out
! 
! ----------------------------------------------------------------------

      subroutine howfar

      implicit none

      include 'include/egs5_h.f'                ! Main EGS "header" file

      include 'include/egs5_epcont.f'    ! COMMONs required by EGS5 code
      include 'include/egs5_stack.f'

      include 'auxcommons/aux_h.f'   ! Auxiliary-code "header" file

                                          ! Auxiliary-code COMMONs
      include 'auxcommons/geoxyz.f'
      include 'auxcommons/instuf.f'

      real*8                                           ! Local variables
     * dist,dnearl

      integer
     * irl,irx,iry,irz

      irl = ir(np)

      if (irl .le. 0) then
        write(6,*) 'Stopped in howfar with irl <= 1'
        stop
      end if

      if (irl .eq. 1) then
        idisc = 1  ! ---------------------------------------------------
        return     ! Particle outside geometry - return to ELECTR/PHOTON
      end if       ! ---------------------------------------------------

!     ---------------------------
!     Get irx, iry and irz indices
!     ---------------------------
      irx=mod(irl-1,imax)
      if (irx.eq.0) irx=imax
      irz=1+(irl-1-irx)/ijmax
      iry=1+(irl-1-irx-(irz-1)*ijmax)/imax

      dnearl = 1.D10 

!     -----------------
!     Check Z-direction
!     -----------------
      dnearl=min(dnearl,(zbound(irz+1)-z(np)),(z(np)-zbound(irz)))
      if (w(np) .gt. 0.0) then
        dist = (zbound(irz+1)-z(np))/w(np)
        if (dist .lt. ustep) then
          ustep=dist
          if (irz .ne. kmax) then
            irnew=irl+ijmax
          else 
            irnew=1
          end if
        end if
      else if (w(np) .lt. 0.0) then
        dist = -(z(np) - zbound(irz))/w(np)
        if (dist .lt. ustep) then
          ustep = dist
          if (irz .ne. 1) then
            irnew=irl-ijmax
          else
            irnew = 1
          end if
        end if
      end if

!     -----------------
!     Check X-direction
!     -----------------
      dnearl=min(dnearl,(xbound(irx+1)-x(np)),(x(np)-xbound(irx)))
      if (u(np) .gt. 0.0) then
          dist = (xbound(irx+1)-x(np))/u(np)
          if (dist .lt. ustep) then
              ustep=dist
              if (irx .ne. imax) then
                  irnew=irl+1
              else 
                  irnew=1
              end if
          end if
      else if (u(np) .lt. 0.0) then
          dist = -(x(np) - xbound(irx))/u(np)
          if (dist .lt. ustep) then
              ustep = dist
              if (irx .ne. 1) then
                  irnew=irl-1
              else
                  irnew = 1
              end if
          end if
      end if

!     -----------------
!     Check Y-direction
!     -----------------
      dnearl=min(dnearl,(ybound(iry+1)-y(np)),(y(np)-ybound(iry)))
      if (v(np) .gt. 0.0) then
          dist = (ybound(iry+1)-y(np))/v(np)
          if (dist .lt. ustep) then
              ustep=dist
              if (iry .ne. jmax) then
                  irnew=irl+imax
              else 
                  irnew=1
              end if
          end if
      else if (v(np) .lt. 0.0) then
          dist = -(y(np) - ybound(iry))/v(np)
          if (dist .lt. ustep) then
              ustep = dist
              if (iry .ne. 1) then
                  irnew=irl-imax
              else
                  irnew = 1
              end if
          end if
      end if

      dnear(np)=dnearl

                                               ! -----------------------
      return                                   ! Return to ELECTR/PHOTON
                                               ! -----------------------
      end

!--------------------------last line of howfar.f------------------------
!-------------------------------encoea.f--------------------------------
! Version:   030831-1300
! Reference: SLAC-265 (p.19-20, Appendix 2)
!-----------------------------------------------------------------------
!23456789|123456789|123456789|123456789|123456789|123456789|123456789|12
!
!     double precision encoea(energy) 
!     Function to evaluate the energy absorption coefficient of air.    
!      (Tables and Graphs oh photon mass attenuation coefficients and 
!      energy-absorption coefficients for photon energies 1 keV to 
!      20 MeV for elements Z=1 to 92 and some dosimetric materials, 
!      S. M. Seltzer and J. H. Hubbell 1995, Japanese Society of 
!      Radiological Technology)
!----------------------------------------------------------------------
      double precision function encoea(energy)

      real*8 hnu(38)/0.001,0.0015,0.002,0.003,0.0032029,0.0032029,
     *           0.004,0.005,0.006,0.008,0.01,0.015,0.02,0.03,0.04,
     *           0.05,0.06,0.08,0.10,0.15,0.2,0.3,0.4,0.5,0.6,0.8,1.0,
     *           1.25,1.5,2.0,3.0,4.0,5.0,6.0,8.0,10.0,15.0,20.0/

      real*8 enmu(38)/3599.,  1188.,  526.2,  161.4,  133.0,  146.0,
     *      76.36, 39.31,  22.70,  9.446,  4.742,  1.334,  0.5389,
     *      0.1537,0.06833,0.04098,0.03041,0.02407,0.02325,0.02496,
     *      0.02672,0.02872,0.02949,0.02966,0.02953,0.02882,0.02789,
     *      0.02666,0.02547,0.02345,0.02057,0.01870,0.01740,0.01647,
     *      0.01525,0.01450,0.01353,0.01311/;
                                                  
      real*8 energy,enm1,hnu1,ene0,slope

      integer i

      if (energy.gt.hnu(38)) then
        encoea=enmu(38)
        return
      end if
      if (energy.lt.hnu(1)) then
        encoea=enmu(1)
        return
      end if
      
      do i=1,38
        if(energy.ge.hnu(i).and.energy.lt.hnu(i+1)) then
          enm1=dlog(enmu(i+1))
          enm0=dlog(enmu(i))
          hnu1=dlog(hnu(i+1))
          hnu0=dlog(hnu(i))

          ene0=dlog(energy)
          slope=(enm1-enm0)/(hnu1-hnu0)
          encoea=exp(enm0+slope*(ene0-hnu0))
          return
        end if
        if(energy.eq.hnu(i+1)) then
          encoea=enmu(i+1)
          return
        end if
      end do

! If sort/interpolation cannot be made, indicate so by writing
! a comment and stopping here.
      write(6,100) energy
100   FORMAT(///,' *****STOPPED IN ENCOEA*****',/,' E=',G15.5,///)
      return
      end
                                                                       
!--------------------------last line of encoea.f------------------------
